using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using de.WWWings.GO;

using System.Linq.Expressions;

namespace de.WWWings.GL
{
 /// <summary>
 /// BL-Manager-Klasse fr Flge
 /// Verwendet im Entity Framework / LINQ-to-Entities
 /// </summary>
 public enum BuchungsFilterStatus
  {
  Alle, NurGebuchte, NurNichtGebuchte
  }
 
 /// <summary>
 /// Managerklasse fr Entitt "Flug"
 /// </summary>
 public class FlugBLManager : IDisposable, de.WWWings.GL.IFlugBLManager 
 {
  /// <summary>
  /// Instanz des EF-Kontextes
  /// </summary>
  WWWings6Entities db = new WWWings6Entities();

  /// <summary>
  /// Wichtige berladung fr Services, 
  /// da bei Serialisierung Verfolgung der Navigationseigenschaften verhindert werden muss!
  /// </summary>
  public FlugBLManager(bool AllowTransparentLazyLoad)
  {
   db.ContextOptions.LazyLoadingEnabled = AllowTransparentLazyLoad;
  }

  /// <summary>
  /// Kontext vernichten, wenn Manager vernichtet wird
  /// </summary>
  public void Dispose()
  {
  db.Dispose();
  }



  public FlugBLManager()
  {
  }

  #region Hilfsroutinen

  // Interne Hilfsroutine: KeySelector fr Sortierung
  private IQueryable<Flug> AddSortingAndPagingToQuery(string Sortierung, IQueryable<Flug> Query, int Start, int Anzahl)
  {
   string SortierungFeldName = Sortierung.Replace(" DESC", "");
   if (SortierungFeldName == "") SortierungFeldName = "Abflugort";
   Expression<Func<Flug, string>> KeySelector = WWWings35_DZ.LINQUTIL.GetKeyStringSelector(SortierungFeldName);

   // Unterscheiden zwischen Sortierrichtung
   if (Sortierung.Contains("DESC"))
   {
    Query = Query.OrderByDescending(KeySelector);
   }
   else
   {
    Query = Query.OrderBy(KeySelector);
   }

   // Blttern und Konvertieren
   Query = Query.Skip(Start).Take(Anzahl);
   return Query;
  }
  #endregion

  #region Count
  /// <summary>
  /// Anzahl der Flge
  /// </summary>
  /// <returns></returns>
  public long CountFluege()
  {
   return (from f in db.Flug select f).Count();
  }

  /// <summary>
  /// Anzahl der Flge
  /// </summary>
  /// <returns></returns>
  public long CountFluegeVonOrt(string Ort)
  {
   return (from f in db.Flug where f.Abflugort == Ort select f).Count();
  }

  /// <summary>
  /// Anzahl der Flge auf einer Route
  /// </summary>
  public long CountFluegeRoute(string Abflugort, string Zielort)
  {
   return (from f in db.Flug where f.Abflugort == Abflugort && f.Zielort == Zielort select f).Count();
  }
  #endregion

  #region Einzelflug
  /// <summary>
  /// Holt einen Flug nach Primrschlssel. Inkludiert alle Passagieren und deren Details sowie den Pilot
  /// </summary
  public Flug GetFlug(long Nr)
  {
   // Flug flug = (from f in db.Flugs where f.FlugNr == Nr select f).SingleOrDefault();

   // fr Unit Test Demos: Single() statt SingleOrDefault()
   Flug flug = (from f in db.Flug where f.FlugNr == Nr select f).FirstOrDefault();
   db.Detach(flug);
   return flug;
  }


  #endregion

  #region Flugmenge
  /// <summary>
  /// Holt Flge von einem Abflugort
  /// </summary>
  public List<Flug> GetFluegeVonOrt(string Ort)
  {
   List<Flug> fluege = (from f in db.Flug where f.Abflugort == Ort select f).ToList();
   return fluege;
  }

  /// <summary>
  /// Holt Flge auf einer Route
  /// </summary>
  public List<Flug> GetFluegeRoute(string Abflugort, string Zielort)
  {
   List<Flug> fluege = (from f in db.Flug where f.Abflugort == Abflugort && f.Zielort == Zielort select f).ToList();
   return fluege;
  }

  /// <summary>
  /// Holt Flge  in einem Zeiraum
  /// </summary>
  public List<Flug> GetFluegeZeitraum(DateTime ZeitraumStart, DateTime ZeitraumEnde)
  {
   return (from flug in db.Flug where flug.Datum >= ZeitraumStart && flug.Datum <= ZeitraumEnde select flug).ToList();
  }


  /// <summary>
  /// Holt Flge auf einer Route in einem Zeiraum
  /// </summary>
  public List<Flug> GetFluegeRouteZeitraum(string Abflugort, string Zielort, DateTime ZeitraumStart, DateTime ZeitraumEnde)
  {
   return (from flug in db.Flug where flug.Abflugort == Abflugort && flug.Zielort == Zielort && flug.Datum >= ZeitraumStart && flug.Datum <= ZeitraumEnde select flug).ToList();
  }

  /// <summary>
  /// Hollt alle Flge
  /// </summary>
  /// <returns></returns>
  public List<Flug> GetFluege()
  {
   List<Flug> fluege = (from f in db.Flug select f).ToList();
   return fluege;
  }

  /// <summary>
  /// Holt alle Flge mit Sortieren und Blttern
  /// </summary>
  public List<Flug> GetFluege_Paging(string Sortierung, int Anzahl, int Start)
  {
   // Basisabfrage nur mit Bedingung
   var Query = (from f in db.Flug select f);
   // Auswerten der Parameter
   Query = AddSortingAndPagingToQuery(Sortierung, Query, Start, Anzahl);
   // Konvertieren
   List<Flug> fluege = Query.ToList();
   return fluege;
  }

  /// <summary>
  /// Holt Flge mit Ortsfilter und Sortieren/Blttern
  /// </summary>
  public List<Flug> GetFluegeVonOrt_Paging(string Ort, string Sortierung, int Anzahl, int Start)
  {
   // Basisabfrage nur mit Bedingung
   var Query = (from f in db.Flug where f.Abflugort == Ort select f);
   // Auswerten der Parameter
   Query = AddSortingAndPagingToQuery(Sortierung, Query, Start, Anzahl);
   // Konvertieren
   List<Flug> fluege = Query.ToList();
   return fluege;
  }

  /// <summary>
  /// Holt Flge mit Routenfilter und Sortieren/Blttern
  /// </summary>
  public List<Flug> GetFluegeRoute_Paging(string Abflugort, string Zielort, string Sortierung, int Anzahl, int Start)
  {
   // Basisabfrage nur mit Bedingung
   var Query = (from f in db.Flug where f.Abflugort == Abflugort && f.Zielort == Zielort select f);
   // Auswerten der Parameter
   Query = AddSortingAndPagingToQuery(Sortierung, Query, Start, Anzahl);
   // Konvertieren
   List<Flug> fluege = Query.ToList();
   return fluege;
  }
  #endregion

  #region Flge ndern


  /// <summary>
  /// Speichern aller nderungen im Kontext
  /// </summary>
  public string SaveFluege()
  {

   // Optional fr Anzeige der Anzahl genderter Daten
   string Rueckgabe = "";
   Rueckgabe += "Gendert: " + db.ObjectStateManager.GetObjectStateEntries(System.Data.EntityState.Modified).Count();
   Rueckgabe += " Neu: " + db.ObjectStateManager.GetObjectStateEntries(System.Data.EntityState.Added).Count();
   Rueckgabe += " Gelscht: " + db.ObjectStateManager.GetObjectStateEntries(System.Data.EntityState.Deleted).Count();
   db.SaveChanges();
   return Rueckgabe;
  }



  /// <summary>
  /// Aufruf, wenn in UI eine Zeile ergnzt wurde
  /// </summary>
  public bool MarkFlugForAdding(Flug flug)
  {
   db.AddToFlug(flug);
   return true;
  }

  /// <summary>
  /// Fr Desktop-UI: Aufruf, wenn in UI eine Zeile gelscht wurde
  /// </summary>
  public bool MarkFlugForDeletion(Flug flug)
  {
   db.DeleteObject(flug);
   return true;
  }
  #endregion

  //public List<Flug> GetFluegeMitStatusFuerPassagier(int PID, BuchungsFilterStatus filterStatus )
  //{
  // List<Flug> alleFluege = GetFluege();
  // Passagier p = new PassagierBLManager().GetPassagier(PID);
  // if (p == null) return alleFluege;

  // // markiere alle Flge, die dieser Passagier gebucht hat
  // foreach (var f in p.Flug) // alle flge dieses Passagiers
  // {
  //  var gesuchterFlug = alleFluege.Where(x => x.FlugNr == f.FlugNr).SingleOrDefault();
  //  gesuchterFlug.IstGebucht = true;
  // }

  // if (filterStatus == BuchungsFilterStatus.NurGebuchte) return alleFluege.Where(x => x.IstGebucht == true).ToList();
  // if (filterStatus == BuchungsFilterStatus.NurNichtGebuchte) return alleFluege.Where(x => x.IstGebucht != true).ToList();

  // return alleFluege;
  //}

  public bool SaveFlug(Flug flug)
  {
   db.Flug.Attach(flug);
   if (flug.Datum == DateTime.MinValue) flug.Datum = DateTime.MaxValue;
   db.ObjectStateManager.ChangeObjectState(flug, System.Data.EntityState.Modified);
   db.SaveChanges();
   return true;
  }


 }
}
