﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Threading.Tasks;

namespace Modell
{
    public abstract class ModelBase : INotifyPropertyChanged, INotifyDataErrorInfo
    {
        private Dictionary<string, List<string>> _errors = new Dictionary<string, List<string>>();

        public IEnumerable GetErrors(string propertyName)
        {
            List<string> errors;
            _errors.TryGetValue(propertyName, out errors);
            return errors;
        }

        [Display(AutoGenerateField = false)]
        public bool HasErrors
        {
            get { return _errors.Values.FirstOrDefault(l => l.Count > 0) != null; }
        }

        [Display(AutoGenerateField = false)]
        public string Errors
        {
            get
            {
                var errorString = "";
                foreach (var error in _errors)
                {
                    error.Value.ForEach(e =>
                    {
                        errorString += error.Key + ": " + e + "\n";
                    });
                }
                if (errorString.Length > 0)
                    errorString = errorString.Substring(0, errorString.Length - 2);
                return errorString;
            }
        }

        private void Validate()
        {
            var props = from p in this.GetType().GetProperties()
                        where p.CanWrite
                        select p.Name;
            foreach (var prop in props)
            {
                List<string> errors;
                if (!_errors.TryGetValue(prop, out errors))
                {
                    errors = new List<string>();
                }
                else
                {
                    errors.Clear();
                }

                var error = this.ValidateProperty(prop);
                if (!string.IsNullOrEmpty(error))
                {
                    errors.Add(error);
                }

                _errors[prop] = errors;
                if (errors.Count > 0)
                {
                    this.OnErrorsChanged(prop);
                }
            }

            this.OnPropertyChanged("Errors");
        }

        protected virtual string ValidateProperty(string propertyName)
        {
            return null;
        }

        public event EventHandler<DataErrorsChangedEventArgs> ErrorsChanged;
        protected virtual void OnErrorsChanged(string propertyName)
        {
            if (ErrorsChanged != null)
                ErrorsChanged(this, new DataErrorsChangedEventArgs(propertyName));
        }

        public event PropertyChangedEventHandler PropertyChanged;
        protected virtual void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            
            if (propertyName != "Errors")
            {
                var waitTask = new Task(() => this.Validate());
                waitTask.Start(); 
            }
        }
    }
}
